/*
 * Copyright 2004-2005 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.strutsel.taglib.html;

import javax.servlet.jsp.JspException;

import org.apache.strutsel.taglib.utils.EvalHelper;
import org.seasar.framework.util.Base64Util;
import org.seasar.framework.util.IntegerConversionUtil;
import org.seasar.framework.util.StringUtil;
import org.seasar.struts.util.S2StrutsContextUtil;

/**
 * @author Satoshi Kimura(SubmitTag)
 * @author Naoki Ichimura(ELSubmitTag)
 */
public class ELSubmitTag extends org.apache.strutsel.taglib.html.ELSubmitTag {

	protected String indexId;
    protected String action;
    
    protected String indexIdExpr;
    protected String actionExpr;

    public int doEndTag() throws JspException {
        setMethodBindingExpression();
        
        try {
            return super.doEndTag();
        } finally {
            release();
        }
    }
    
    /**
     * @return Returns the indexId.
     */
    public String getIndexId() {
        return this.indexId;
    }
    
    /**
     * @param indexId The indexName to set.
     */
    public void setIndexId(String indexId) {
        this.indexId = indexId;

        if (indexId != null) {
            super.indexed = true;
        }
    }

    public String getAction() {
        return this.action;
    }

    public void setAction(String action) {
        this.action = action;
    }

    protected void prepareIndex(StringBuffer handlers, String name) throws JspException {
        if (this.indexId == null) {
            super.prepareIndex(handlers, name);
            return;
        }
        Object index = super.pageContext.getAttribute(this.indexId);
        handlers.append("[");
        handlers.append(IntegerConversionUtil.toPrimitiveInt(index));
        handlers.append("]");
    }
    
    protected void setMethodBindingExpression() {
        if (StringUtil.isEmpty(super.property)) {
            super.property = Base64Util.encode(this.action.getBytes());
        }
        String val = super.value;
        if(val == null) {
            val = super.text;
        }
        if(val == null) {
            val = super.getDefaultValue();
        }
        S2StrutsContextUtil.setMethodBindingExpression(super.property, val, this.action);
    }

    public String getIndexIdExpr() {
        return this.indexIdExpr;
    }
    
    public void setIndexIdExpr(String indexIdExpr) {
        this.indexIdExpr = indexIdExpr;

        if (indexIdExpr != null) {
            super.indexed = true;
        }
    }

    public String getActionExpr() {
		return this.actionExpr;
	}

	public void setActionExpr(String actionExpr) {
		this.actionExpr = actionExpr;
	}

    public void release() {
        super.release();
        setIndexId(null);
        setAction(null);
        setIndexIdExpr(null);
        setActionExpr(null);
    }

    public int doStartTag() throws JspException {
		evaluateExpressions();
		return (super.doStartTag());
	}
    
	private void evaluateExpressions() throws JspException {
		String string = null;

		if ((string = EvalHelper.evalString("indexId", getIndexIdExpr(), this,
				pageContext)) != null) {
			setIndexId(string);
		}
		if ((string = EvalHelper.evalString("action", getActionExpr(), this,
				pageContext)) != null) {
			setAction(string);
		}
	}

}